"""
.. codeauthor:: Nathan Hertz <nhertz@nrao.edu>

Workspaces testing utilities

Put things here if you'd like them to be available to any given test in the entire system

HOW TO USE:
1. In any given pytest-powered test, add this line below the imports:
pytest_plugins = ["testing.utils.conftest"]
2. All fixtures within this file will now be available for use
3. If you want a non-fixture item from here, you'll have to do a normal Python import
"""
from typing import List
from unittest.mock import patch

import pytest
from alchemy_mock.mocking import UnifiedAlchemyMagicMock

from workspaces.capability.schema import Capability
from workspaces.capability.schema_interfaces import CapabilityRequestIF
from workspaces.capability.services.capability_info import CapabilityInfo
from workspaces.capability.services.capability_service import CapabilityService
from workspaces.workflow.services.workflow_info import WorkflowInfo
from workspaces.workflow.services.workflow_service import WorkflowService


@pytest.fixture(scope="session")
def mock_capability_info() -> CapabilityInfo:
    """
    Pytest fixture providing a mock CapabilityInfo object with a test capability; mocked object can be used
    like a normal CapabilityInfo instance
    TODO: Use UnifiedAlchemyMagicMock's data parameter to add test capabilities

    :return: Mocked CapabilityInfo
    """
    mocked_capability_info = CapabilityInfo(UnifiedAlchemyMagicMock())
    null_capability = Capability(
        name="null", steps="prepare-and-run-workflow null\nawait-workflow", max_jobs=2
    )
    # test_capability = Capability(name="test", steps="await-qa", max_jobs=777)
    mocked_capability_info.save_entity(null_capability)
    # mocked_capability_info.save_entity(test_capability)

    return mocked_capability_info


@pytest.fixture(scope="session")
def mock_capability_service(
    mock_capability_info: CapabilityInfo,
    mock_workflow_service: WorkflowService,
) -> CapabilityService:
    """
    Pytest fixture providing a mock CapabilityService object with no threaded event listener creation

    :return: Mocked CapabilityService
    """
    with patch("threading.Thread"):
        with patch("threading.Thread.start"):
            yield CapabilityService(mock_capability_info, mock_workflow_service)


@pytest.fixture(scope="session")
def mock_workflow_info() -> WorkflowInfo:
    """
    Pytest fixture providing a mock WorkflowInfo object; mocked object can be used
    like a normal WorkflowInfo instance

    :return: Mocked WorkflowInfo
    """
    return WorkflowInfo(UnifiedAlchemyMagicMock())


@pytest.fixture(scope="session")
def mock_workflow_service(mock_workflow_info: WorkflowInfo) -> WorkflowService:
    """
    Pytest fixture providing a mock WorkflowService object

    :return: Mocked WorkflowService
    """
    with patch("threading.Thread"):
        with patch("threading.Thread.start"):
            yield WorkflowService(mock_workflow_info)


@pytest.fixture(scope="session")
def mock_capability_request(
    mock_capability_service: CapabilityService,
) -> List[CapabilityRequestIF]:
    """
    Pytest fixture adding two mock CapabilityRequest objects that can be accessed from the mock_capability_info

    :param mock_capability_service: Mocked CapabilityService
    :return: Mocked CapabilityRequest
    """
    requests = []
    r1 = mock_capability_service.create_request("null", ["-g"])
    r1.id = 0
    requests.append(r1)
    r2 = mock_capability_service.create_request("test")
    r2.id = 1
    requests.append(r2)
    return requests
